#ifndef WCBE_APP_H_
#define WCBE_APP_H_

G_BEGIN_DECLS

extern const gchar wcbe_input_stage_pipeline_name[];

typedef enum {
    WCBE_DUMP_NONE,
    WCBE_DUMP_BINARY,
    WCBE_DUMP_ASCII
} wcbe_dump_format_t;

typedef enum {
    WCBE_FLOW_OFF,
    WCBE_FLOW_ON
} wcbe_flow_t;

typedef enum {
    WCBE_CONFIG_CREATE_MSG = 0,
    WCBE_CONFIG_ACTIVATE_MSG,
    WCBE_CONFIG_DESTROY_MSG,
    WCBE_CONFIG_LIST_MSG,
    WCBE_SRC_MSG,
    WCBE_LIVE_TS_MSG,
    WCBE_FRAMEDUMP_MSG,
    WCBE_FLOW_MSG,
    WCBE_RATE_MSG,
    WCBE_NAME_MSG,
    WCBE_QUIT_MSG,
    WCBE_PIPELINE_LIST_MSG,
    WCBE_ELEMENT_LIST_MSG,
    WCBE_PROPERTY_LIST_MSG,
    WCBE_PROPERTY_MSG,
    WCBE_PAD_LIST_MSG,
    WCBE_PAD_CAPS_MSG,
    WCBE_PAD_PEER_MSG,
    WCBE_NUM_MSG_TYPES
} wcbe_msg_type_t;

typedef enum {
    WCBE_REQUEST = 0,
    WCBE_RESPONSE
} wcbe_msg_direction_t;

typedef enum {
    WCBE_COMMAND=0,
    WCBE_QUERY,
    WCBE_NUM_MSG_INTENTS
} wcbe_msg_intent_t;

typedef struct wcbe_msg {
    wcbe_msg_type_t type;
    wcbe_msg_direction_t direction;
    wcbe_msg_intent_t intent;
    guint id;
} wcbe_msg_t;

#define WCBE_MAX_NAME_LEN 256

typedef struct wcbe_name_msg {
    wcbe_msg_t hdr;
    gchar value[WCBE_MAX_NAME_LEN];
} wcbe_name_msg_t;
typedef struct wcbe_config_create_msg {
    wcbe_msg_t hdr;
    gchar name[WCBE_MAX_NAME_LEN];
    guint desc_len;
    gchar desc[0];
} wcbe_config_create_msg_t;
typedef struct wcbe_config_activate_msg {
    wcbe_msg_t hdr;
    gchar name[WCBE_MAX_NAME_LEN];
} wcbe_config_activate_msg_t;
typedef struct wcbe_config_destroy_msg {
    wcbe_msg_t hdr;
    gchar name[WCBE_MAX_NAME_LEN];
} wcbe_config_destroy_msg_t;
typedef struct wcbe_config_list_msg {
    wcbe_msg_t hdr;
    guint list_len;
    gchar list[0];
} wcbe_config_list_msg_t;
typedef struct wcbe_src_msg {
    wcbe_msg_t hdr;
    gchar name[WCBE_MAX_NAME_LEN];
} wcbe_src_msg_t;
typedef struct wcbe_live_ts_msg {
    wcbe_msg_t hdr;
    gboolean state;
} wcbe_live_ts_msg_t;
typedef struct wcbe_framedump_msg {
    wcbe_msg_t hdr;
    wcbe_dump_format_t format;
} wcbe_framedump_msg_t;
typedef struct wcbe_flow_msg {
    wcbe_msg_t hdr;
    wcbe_flow_t state;
} wcbe_flow_msg_t;
typedef struct wcbe_rate_msg {
    wcbe_msg_t hdr;
    guint fps;
} wcbe_rate_msg_t;
typedef struct wcbe_quit_msg {
    wcbe_msg_t hdr;
    guint reserved;
} wcbe_quit_msg_t;
typedef struct wcbe_pipeline_list_msg {
    wcbe_msg_t hdr;
    gchar configuration[WCBE_MAX_NAME_LEN];
    guint list_len;
    gchar list[0];
} wcbe_pipeline_list_msg_t;
typedef struct wcbe_element_list_msg {
    wcbe_msg_t hdr;
    gchar pipeline[WCBE_MAX_NAME_LEN];
    guint list_len;
    gchar list[0];
} wcbe_element_list_msg_t;
typedef struct wcbe_property_list_msg {
    wcbe_msg_t hdr;
    gchar element[WCBE_MAX_NAME_LEN];
    guint list_len;
    gchar list[0];
} wcbe_property_list_msg_t;
typedef struct wcbe_property_msg {
    wcbe_msg_t hdr;
    gchar property[WCBE_MAX_NAME_LEN];
    gchar type[WCBE_MAX_NAME_LEN];
    guint value_len;
    gchar value[0];
} wcbe_property_msg_t;
typedef struct wcbe_pad_list_msg {
    wcbe_msg_t hdr;
    gchar element[WCBE_MAX_NAME_LEN];
    guint list_len;
    gchar list[0];
} wcbe_pad_list_msg_t;
typedef struct wcbe_pad_caps_msg {
    wcbe_msg_t hdr;
    gchar pad[WCBE_MAX_NAME_LEN];
    gchar caps[WCBE_MAX_NAME_LEN]; // TODO: make this of variable length
} wcbe_pad_caps_msg_t;
typedef struct wcbe_pad_peer_msg {
    wcbe_msg_t hdr;
    gchar pad[WCBE_MAX_NAME_LEN];
    gchar peer[WCBE_MAX_NAME_LEN];
} wcbe_pad_peer_msg_t;

G_END_DECLS

#endif /* WCBE_APP_H_ */
