#! /usr/bin/env python

from optparse import OptionParser
import os.path
import os
import readline
import atexit
from PipelineMC import *

def selword(words, text, state):
    try:
        result = [w for w in words if w.startswith(text)][state]
    except:
        result = None
    return result

def completer(text, state):

    result = None
    begidx = readline.get_begidx()
    if begidx == 0:
        result = selword(('get', 'set', 'quit', 'kill', 'create', 'destroy'),
                         text, state)
    elif begidx == 4:
        attributes = ('config', 'flow', 'framedump', 'live_ts',
                      'name', 'src', 'rate', 'property')
        if readline.get_line_buffer().startswith('get'):
            attributes += ('pipelines', 'elements', 'properties', 'pads',
                           'caps', 'configs', 'peer')
        result = selword(attributes, text, state)
    else:
        line = readline.get_line_buffer()
        if line.startswith('set flow'):
            result = selword(tuple(Flow.strs()), text, state)
        elif line.startswith('set framedump'):
            result = selword(tuple(DumpFormat.strs()), text, state)
        elif line.startswith('set live_ts'):
            result = selword(tuple(LiveTs.strs()), text, state)
    return result

def parse_input(text, be):
    words = text.split(' ', 3)
    if len(words) > 0:
        msg = None
        kill_be = False
        if words[0] == 'get':
            if len(words) > 1:
                if words[1] == 'config' or words[1] == 'configs':
                    msg = ConfigListMsg(intent=MsgIntent.query)
                elif words[1] == 'src':
                    msg = SrcMsg(intent=MsgIntent.query)
                elif words[1] == 'live_ts':
                    msg = LiveTsMsg(intent=MsgIntent.query)
                elif words[1] == 'flow':
                    msg = FlowMsg(intent=MsgIntent.query)
                elif words[1] == 'framedump':
                    msg = FramedumpMsg(intent=MsgIntent.query)
                elif words[1] == 'name':
                    msg = NameMsg(intent=MsgIntent.query)
                elif words[1] == 'rate':
                    msg = RateMsg(intent=MsgIntent.query)
                elif words[1] == 'pipelines':
                    if len(words) > 2:
                        msg = PipelineListMsg(config=words[2])
                    else:
                        print "Configuration name argument of " \
                              "'get pipelines' command is missing"
                elif words[1] == 'elements':
                    if len(words) > 2:
                        msg = ElementListMsg(pipeline=words[2])
                    else:
                        print "Pipeline name argument of " \
                              "'get elements' command is missing"
                elif words[1] == 'properties':
                    if len(words) > 2:
                        msg = PropertyListMsg(element=words[2])
                    else:
                        print "Element name argument of " \
                              "'get properties' command is missing"
                elif words[1] == 'pads':
                    if len(words) > 2:
                        msg = PadListMsg(element=words[2])
                    else:
                        print "Element name argument of " \
                              "'get pads' command is missing"
                elif words[1] == 'caps':
                    if len(words) > 2:
                        msg = PadCapsMsg(pad=words[2])
                    else:
                        print "Pad name argument of " \
                              "'get caps' command is missing"
                elif words[1] == 'peer':
                    if len(words) > 2:
                        msg = PadPeerMsg(pad=words[2])
                    else:
                        print "Pad name argument of " \
                              "'get peer' command is missing"
                elif words[1] == 'property':
                    if len(words) > 2:
                        msg = PropertyMsg(prop=words[2])
                    else:
                        print "Property name argument of " \
                              "'get property' command is missing"
            else:
                print "Argument of 'get' command is missing"
        elif words[0] == 'set':
            if len(words) >= 3:
                if words[1] == 'config':
                    msg = ConfigActivateMsg(intent=MsgIntent.command,
                                            name=words[2])
                elif words[1] == 'src':
                    msg = SrcMsg(intent=MsgIntent.command, name=words[2])
                elif words[1] == 'live_ts':
                    try:
                        msg = LiveTsMsg(intent=MsgIntent.command,
                                        state=LiveTs(words[2]))
                    except:
                        pass
                elif words[1] == 'flow':
                    try:
                        msg = FlowMsg(intent=MsgIntent.command,
                                      state=Flow(words[2]))
                    except:
                        pass
                elif words[1] == 'framedump':
                    try:
                        msg = FramedumpMsg(intent=MsgIntent.command,
                                           format=DumpFormat(words[2]))
                    except:
                        pass
                elif words[1] == 'rate':
                    try:
                        msg = RateMsg(intent=MsgIntent.command,
                                      rate=int(words[2]))
                    except:
                        pass
                elif words[1] == 'name':
                    parts = words[2].split(':')
                    res = False
                    if len(parts) == 1:
                        res = be.add_pipeline(alias=words[2], name=parts[0])
                    elif len(parts) == 2:
                        res = be.add_pipeline(alias=words[2], host=parts[0],
                                              port=int(parts[1]))
                    elif len(parts) == 3:
                        res = be.add_pipeline(alias=words[2], name=parts[2],
                                              host=parts[0],
                                              port=int(parts[1]))
                    if res:
                        msg = NameMsg(intent=MsgIntent.query)
                elif words[1] == 'property':
                    try:
                        msg = PropertyMsg(prop=words[2])
                        msg = be.mc_transaction(msg)
                        msg = PropertyMsg(intent=MsgIntent.command,
                                          valtype=msg.valtype,
                                          prop=msg.prop,
                                          value=words[3])
                    except:
                        pass
            else:
                print "Argument of 'set' command is missing"
        elif words[0] == 'kill':
            msg = QuitMsg()
            kill_be = True
        elif words[0] == 'create':
            if len(words) == 3:
                conffile = open(words[2])
                msg = ConfigCreateMsg(intent=MsgIntent.command,
                                      name=words[1], desc=conffile.read())
                conffile.close()
            else:
                print "Argument of 'create' command is missing"
        elif words[0] == 'destroy':
            if len(words) > 1:
                msg = ConfigDestroyMsg(intent=MsgIntent.command, name=words[1])
            else:
                print "Argument of 'destroy' command is missing"
        elif words[0] != '' and words[0] != 'quit':
            print "Unrecognized command"
        if msg is not None:
            if be.name is not None:
                intent = msg.intent
                msg = be.mc_transaction(msg)
                if intent == MsgIntent.query:
                    if msg is not None:
                        if words[1] == 'config':
                            if len(msg.configs.active) > 0:
                                print "Active configuration name: '%s'" \
                                      % msg.configs.active[0]
                            elif len(msg.configs.active_retired) > 0:
                                print "Active configuration name: '%s'" \
                                      % msg.configs.active_retired[0]
                            else:
                                print "No active configuration"
                        elif words[1] == 'configs':
                            print 'Active configuration:'
                            if len(msg.configs.active) == 0 \
                                    and len(msg.configs.active_retired) == 0:
                                print '\t(none)'
                            if len(msg.configs.active) == 1:
                                print '\t%s' % msg.configs.active[0]
                            if len(msg.configs.active_retired) == 1:
                                print '\t%s (retired)' % \
                                    msg.configs.active_retired[0]
                            print 'Inactive configurations:'
                            if len(msg.configs.inactive) > 0:
                                for c in msg.configs.inactive:
                                    print '\t%s' % c
                            if len(msg.configs.pending) > 0:
                                print 'Pending configuration actions:'
                                for p in msg.configs.pending:
                                    print '\t %s' % p
                        elif words[1] == 'src':
                            print "Backend source name: '%s'" % msg.name
                        elif words[1] == 'live_ts':
                            print msg.state
                        elif words[1] == 'flow':
                            print msg.state
                        elif words[1] == 'framedump':
                            print msg.format
                        elif words[1] == 'rate':
                            print "File data frame rate: %d" % msg.rate
                        elif words[1] == 'name':
                            print "Pipeline instance name: '%s'" % msg.name
                        elif words[1] == 'pipelines':
                            print "Pipelines in configuration '%s':" % \
                                  msg.config
                            for p in msg.pipelines:
                                print '\t%s' % p
                        elif words[1] == 'elements':
                            print "Elements in pipeline '%s':" % msg.pipeline
                            for e in msg.elements:
                                print '\t%s' % e
                        elif words[1] == 'properties':
                            print "Properties of element '%s':" % msg.element
                            for p in msg.properties:
                                print '\t%s' % p
                        elif words[1] == 'pads':
                            print "Pads of element '%s':" % msg.element
                            for p in msg.pads:
                                print '\t%s' % p
                        elif words[1] == 'caps':
                            print "Pad caps of '%s':" % msg.pad
                            print '\t%s' % msg.caps
                        elif words[1] == 'peer':
                            print "Peer of pad '%s':" % msg.pad
                            print '\t%s' % msg.peer
                        elif words[1] == 'property':
                            print "Property '%s' value (%s):" % \
                                  (msg.prop, msg.valtype)
                            print '\t%s' % str(msg.value)
                elif kill_be:
                    be.name = None
            else:
                print "Pipeline instance name not set..."
                print "Use 'set name <name>' to select or create a "\
                      "pipeline instance"
    return

def show_confs(configs, tag):
    configs = [cnf[1:-1] for cnf in configs if cnf[:1] == tag]
    if len(configs) > 0:
        for c in configs:
            print '\t%s' % c
    else:
        print '\t(none)'
    return

class Client (PipelineClient):

    def __init__(self, sockdir, with_udp=True, with_unix=True):
        super(Client, self).__init__(sockdir=sockdir, with_udp=with_udp,
                                     with_unix=with_unix)
        self.name = None
        return

    def add_pipeline(self, alias, name=None, host=None, port=None,
                     allow_new=True):
        rc = super(Client, self).add_pipeline(alias=alias, name=name,
                                              host=host, port=port,
                                              allow_new=allow_new)
        if rc:
            self.name=alias
        return rc

    def mc_transaction(self, msg, alias=None):
        if alias is None:
            alias = self.name
        return super(Client, self).mc_transaction(alias=alias, msg=msg)

def main():

    parser = OptionParser(description='Console for interactive ' \
                          'control of WIDAR correlator backend system ' \
                          'on compute nodes.')
    parser.add_option('--name', '-n', action='store', type='string',
                      dest='name',
                      help='Pipeline instance name ' \
                      '(host:port:name for remote, otherwise local)')
    parser.add_option('--sockdir', '-s', action='store', type='string',
                      dest='sockdir',
                      help="Directory for Unix domain sockets")
    (options, args) = parser.parse_args()

    if options.sockdir is not None:
        sockdir = options.sockdir
    else:
        if os.environ.has_key('TMPDIR'):
            sockdir = os.environ['TMPDIR']
        elif os.environ.has_key('TMP'):
            sockdir = os.environ['TMP']
        elif os.environ.has_key('TEMP'):
            sockdir = os.environ['TEMP']
        else:
            sockdir = '/tmp'

    be = Client(sockdir=sockdir)
    if options.name is not None and options.name != '':
        parse_input('set name %s' % options.name, be)

    readline.set_history_length(100)
    readline.set_completer(completer)
    readline.parse_and_bind('tab: complete')
    histfile = None
    def histfile_cleanup():
        if histfile is not None:
            readline.write_history_file(histfile)
    atexit.register(histfile_cleanup)
    try:
        histfile = os.path.join(os.environ['HOME'], '.wcbehist')
        readline.read_history_file(histfile)
    except IOError:
        print 'Failed to open history file; command history will not be saved.'
        pass

    inp = ''
    while inp != 'quit':
        if be.name is not None:
            prompt = '%s> ' % be.name
        else:
            prompt = '> '
        try:
            inp = raw_input(prompt)
            parse_input(inp.strip(), be)
            try:
                os.waitpid(-1, os.WNOHANG) # TODO: improve this
            except:
                pass
        except EOFError:
            print
            inp = 'quit'

if __name__ == '__main__':
    main()
