#ifndef WCBE_PRIVATE_H_
#define WCBE_PRIVATE_H_

#include <glib.h>
#include <gst/gst.h>

#include <wcbe_app.h>
#include <enums.h>
#include <sys/un.h>

G_BEGIN_DECLS

/* This module can be compiled to implement the control socket under
 * various protocols. The following selects the implementation.
 */
#undef CONTROL_SOCK_UNIX
#define CONTROL_SOCK_UDP 1

typedef struct configuration_ configuration_t;

typedef struct pipeline {
    GMainLoop *loop;
    gchar *name;
    gchar *src;
    gboolean live_ts;
    wcbe_dump_format_t framedump;
    wcbe_flow_t flow;
    guint fps;
    gchar *sockdir;
    gint sockfd;
    GstElement *is_pipeline;
    GstElement *old_is_pipeline;
    GHashTable *configs;
    GSList *old_configs;
    GSList *pending_configs;
    configuration_t *config;
    GSList *old_ls_pipelines;
    GIOChannel *lsp_channel;
    guint lsp_source;
    gint lsp_sockfd;
    GHashTable *msg_callbacks;
    guint bus_watch;
#if CONTROL_SOCK_UDP
    gchar *ctladdr;
#endif
} pipeline_t;

typedef enum {CONFIG_STATE_INITIALIZING, CONFIG_STATE_READY,
              CONFIG_STATE_NOT_READY, CONFIG_STATE_FINALIZING} config_state_t;

struct configuration_ {
    gchar *name;
    gchar *desc;
    gchar *dir;
    GSList *ls_pipelines;
    gboolean destroy;
    pipeline_t *pipeline;
    config_state_t state;
};

typedef struct ls_pipeline {
    gchar *name;
    struct sockaddr_un sockaddr;
    gchar *sockname;
    GPid pid;
    gchar **argv;
    guint spawn_count;
    pipeline_state_t state;
    pipeline_t *pipeline;
    configuration_t *config;
    gboolean destroy;
    guint exit_timeout;
} ls_pipeline_t;

typedef struct pending_config {
    enum {PENDING_CREATE, PENDING_ACTIVATE, PENDING_DESTROY} action;
    configuration_t *config;
} pending_config_t;

extern gboolean wcbe_create_config(pipeline_t *pipeline,
                                   configuration_t *config);
extern void wcbe_insert_pending_conf(pipeline_t *pipeline, guint action,
                                     configuration_t *config);
extern void wcbe_cleanup_config(configuration_t *config);
extern gboolean wcbe_activate_config(pipeline_t *pipeline,
                                     configuration_t *config);
extern gboolean wcbe_delayed_activate_config(gpointer data);
extern void wcbe_destroy_config(pipeline_t *pipeline, configuration_t *config);
extern void wcbe_destroy_pipeline(pipeline_t *pipeline);
extern void wcbe_create_pipeline(pipeline_t *pipeline);
extern void wcbe_start_pipeline(pipeline_t *pipeline, gboolean in_main_loop);
extern void wcbe_stop_pipeline(pipeline_t *pipeline, gboolean in_main_loop);
extern ls_pipeline_t *wcbe_find_ls_pipeline(GSList *list, const gchar *name);
extern GSList *wcbe_find_pending_config(GSList *pending_configs,
                                        const gchar *name);

#define DELAYED_ACTIVATE_INTERVAL 250

#define DEFAULT_PIPELINE_NAME NULL
#define DEFAULT_PIPELINE_SRC NULL
#define DEFAULT_PIPELINE_CONFIGS NULL
#define DEFAULT_PIPELINE_OLD_CONFIGS NULL
#define DEFAULT_PIPELINE_PENDING_CONFIGS NULL
#define DEFAULT_PIPELINE_CONFIG NULL
#define DEFAULT_PIPELINE_LIVE_TS FALSE
#define DEFAULT_PIPELINE_FRAMEDUMP WCBE_DUMP_NONE
#define DEFAULT_PIPELINE_FLOW WCBE_FLOW_OFF
#define DEFAULT_PIPELINE_FPS 100
#define DEFAULT_PIPELINE_SOCKDIR NULL
#define DEFAULT_PIPELINE_LSP_CHANNEL NULL
#define DEFAULT_PIPELINE_MSG_CALLBACKS NULL

G_END_DECLS

#endif /* WCBE_PRIVATE_H_ */
